from typing import Optional

from genologics.entities import Artifact, Process
from pydantic import Field, validator

from cg_lims.models.api.master_steps.base_step import get_artifact_udf, BaseStep
from cg_lims.get.artifacts import get_latest_analyte


class PoolsamplesforhybridizationTWIST(BaseStep):
    "pool samples TWIST v2"
    artifact: Optional[Artifact]
    process: Optional[Process]
    amount_of_sample: Optional[str] = Field(None, alias="Total Amount (ng)")

    @validator("artifact", always=True)
    def get_artifact(cls, v, values):
        try:
            return get_latest_analyte(
                lims=values.get("lims"),
                sample_id=values.get("sample_id"),
                process_types=["pool samples TWIST v2"],
            )
        except:
            return None

    @validator("process", always=True)
    def get_process(cls, v, values):
        if values.get("artifact"):
            return values.get("artifact").parent_process
        return None

    @validator("amount_of_sample", always=True)
    def get_amount_of_sample(cls, v, values):
        return get_artifact_udf(values.get("artifact"), "Total Amount (ng)")



from typing import Optional

from genologics.lims import Lims
from pydantic import Field, BaseModel
from cg_lims.objects import BaseAnalyte
from cg_lims.models.arnold.base_step import BaseStep


class ArtifactUDFs(BaseModel):
    bead_purification_size_bp: Optional[int] = Field(None, alias="Size (bp)")
    bead_purification_concentration: Optional[float] = Field(None, alias="Concentration")


class ProcessUDFs(BaseModel):
    lot_nr_etoh_bead_purification_post_hyb: Optional[str] = Field(None, alias="Ethanol")
    lot_nr_h2o_bead_purification_post_hyb: Optional[str] = Field(None, alias="Nuclease free water")
    bead_purification_post_hyb_method: Optional[str] = Field(None, alias="Method document")
    binding_and_purification_beads: Optional[str] = Field(
        None, alias="Twist Binding and Purification beads"
    )


class ArnoldStep(BaseStep):
    artifact_udfs: ArtifactUDFs
    process_udfs: ProcessUDFs

    class Config:
        allow_population_by_field_name = True


def get_bead_purification_twist(lims: Lims, sample_id: str, prep_id: str) -> ArnoldStep:
    analyte = BaseAnalyte(
        lims=lims,
        sample_id=sample_id,
        process_type="Target enrichment TWIST v1",
    )

    return ArnoldStep(
        **analyte.base_fields(),
        process_udfs=ProcessUDFs(**analyte.process_udfs()),
        artifact_udfs=ArtifactUDFs(**analyte.artifact_udfs()),
        sample_id=sample_id,
        prep_id=prep_id,
        step_type="bead_purification",
        workflow="TWIST",
    )
